// ------------------------------------------------------------
// Machinery project - math library
// ------------------------------------------------------------
// (c) 2000 by Sascha 'EvilOne' Salevsky
// E-mail: Sascha.Salevsky@Ibykus.de
// ------------------------------------------------------------

// Constructors

inline Matrix3::Matrix3()
{}

inline Matrix3::Matrix3(const float* other)
{
	for(int i = 0; i < 9; i++) M[i] = other[i];
}

inline Matrix3::Matrix3(const Matrix3& other)
{
	for(int i = 0; i < 9; i++) M[i] = other.M[i];
}

// Assignment

inline Matrix3& Matrix3::operator=(const Matrix3& other)
{
	for(int i = 0; i < 9; i++) M[i] = other.M[i];
	return *this;
}

inline void Matrix3::Assign(const Matrix3& other)
{
	for(int i = 0; i < 9; i++) M[i] = other.M[i];
}

inline void Matrix3::Assign(const float* other)
{
	for(int i = 0; i < 9; i++) M[i] = other[i];
}

// Casting

inline Matrix3::operator const float*() const
{
	return M;
}

inline Matrix3::operator float*()
{
	return M;
}

// Simple Methods

inline Matrix3& Matrix3::SetIdentity()
{
	for(int i = 0; i < 9; i++) M[i] = Matrix3::Identity_Matrix[i];
	return *this;
}

inline Matrix3& Matrix3::SetZero()
{
	for(int i = 0; i < 9; i++) M[i] = 0.0f;
	return *this;
}

inline Matrix3& Matrix3::Mul(const Matrix3& other)
{
	Matrix3 tmp;
	tmp.Assign(*this);
	Mul(tmp, other);
	return *this;
}

inline Matrix3& Matrix3::Mul(const Matrix3& m1, const Matrix3& m2)
{
	const float *b = m1.M;
	const float *a = m2.M;
	float *prod = M;

	for(int i = 0; i < 3; i++)
	{
		float a0 = a[i];
		float a1 = a[i + 3];
		float a2 = a[i + 6];
		prod[i]     = a0 * b[0] + a1 * b[1] + a2 * b[2];
		prod[i + 3] = a0 * b[3] + a1 * b[4] + a2 * b[5];
		prod[i + 6] = a0 * b[6] + a1 * b[7] + a2 * b[8];
	}
	return *this;
}

inline Matrix3& Matrix3::Mul(float f)
{
	for(int i = 0; i < 9; i++) M[i] *=f;
	return *this;
}

inline Matrix3& Matrix3::Mul(const Matrix3& other, float f)
{
	const float* m = other.M;
	for(int i = 0; i < 9; i++) M[i] = m[i] * f;
	return *this;
}

inline Matrix3& Matrix3::Div(float f)
{
	f = 1.0f / f;
	for(int i = 0; i < 9; i++) M[i] *=f;
	return *this;
}

inline Matrix3& Matrix3::Div(const Matrix3& other, float f)
{
	f = 1.0f / f;
	const float* m = other.M;
	for(int i = 0; i < 9; i++) M[i] = m[i] * f;
	return *this;
}

// Operators

inline Matrix3& Matrix3::operator*=(const Matrix3& other)
{
	Matrix3 tmp;
	tmp.Assign(*this);
	Mul(tmp, other);
	return *this;
}

inline Matrix3 Matrix3::operator*(const Matrix3& other) const
{
	Matrix3 tmp;
	tmp.Mul(*this, other);
	return tmp;
}

// Transformations

inline Matrix3& Matrix3::Translate(float tx, float ty)
{
	M[6] = M[0] * tx + M[3] * ty + M[6];
	M[7] = M[1] * tx + M[4] * ty + M[7];
	M[8] = M[2] * tx + M[5] * ty + M[8];
	return *this;
}

inline Matrix3& Matrix3::SetTranslate(float tx, float ty)
{
	SetIdentity();
	M[6] = tx;
	M[7] = ty;
	return *this;
}

inline Matrix3& Matrix3::Scale(float sx, float sy)
{
	M[0] *= sx; M[3] *= sy;
	M[1] *= sx; M[4] *= sy;
	M[2] *= sx; M[5] *= sy;
	return *this;
}

inline Matrix3& Matrix3::SetScale(float sx, float sy)
{
	M[0] = sx; M[4] = sy; M[8] = 1.0f;
	M[1] = M[2] = M[3] = M[5] = M[6] = M[7] = 0.0f;
	return *this;
}

inline Matrix3& Matrix3::Rotate(float r)
{
	float c = CosD(r), s = SinD(r);
	Matrix3 tmp;
	tmp.SetIdentity();
	tmp.M[0] = c;
	tmp.M[1] = -s;
	tmp.M[3] = s;
	tmp.M[4] = c;
	Mul(tmp);
	return *this;
}

inline Matrix3& Matrix3::SetRotate(float r)
{
	float c = CosD(r), s = SinD(r);
	Matrix3 tmp;
	SetIdentity();
	M[0] = c;
	M[1] = -s;
	M[3] = s;
	M[4] = c;
	return *this;
}
